@@ -27,9 +27,11 @@
 #include <X11/Xatom.h>
 #include <string.h>
 
-void meta_workspace_queue_calc_showing  (MetaWorkspace *workspace);
-
-static void set_active_space_hint      (MetaScreen *screen);
+void meta_workspace_queue_calc_showing      (MetaWorkspace *workspace);
+static void set_active_space_hint           (MetaScreen *screen);
+static void meta_workspace_focus_mru_window (MetaWorkspace *workspace,
+                                             MetaWindow    *not_this_one,
+                                             Time           timestamp);
 
 static void
 maybe_add_to_list (MetaScreen *screen, MetaWindow *window, gpointer data)
@@ -269,7 +271,8 @@ meta_workspace_queue_calc_showing  (MetaWorkspace *workspace)
 
 void
 meta_workspace_activate_with_focus (MetaWorkspace *workspace,
-                                    MetaWindow    *focus_this)
+                                    MetaWindow    *focus_this,
+                                    Time           timestamp)
 {
   MetaWorkspace *old;
   MetaWindow *move_window;
@@ -321,8 +324,7 @@ meta_workspace_activate_with_focus (MetaWorkspace *workspace,
 
   if (focus_this)
     {
-      meta_window_focus (focus_this,
-                         meta_display_get_current_time (focus_this->display));
+      meta_window_focus (focus_this, timestamp);
       meta_window_raise (focus_this);
     }
   else if (move_window)
@@ -332,15 +334,15 @@ meta_workspace_activate_with_focus (MetaWorkspace *workspace,
   else
     {
       meta_topic (META_DEBUG_FOCUS, "Focusing default window on new workspace\n");
-      meta_workspace_focus_default_window (workspace, NULL);
+      meta_workspace_focus_default_window (workspace, NULL, timestamp);
     }
 }
 
 void
-meta_workspace_activate (MetaWorkspace *workspace)
+meta_workspace_activate (MetaWorkspace *workspace,
+                         Time           timestamp)
 {
-  meta_workspace_activate_with_focus (workspace,
-                                      NULL);
+  meta_workspace_activate_with_focus (workspace, NULL, timestamp);
 }
 
 int
@@ -789,10 +791,18 @@ meta_workspace_get_name (MetaWorkspace *workspace)
 
 void
 meta_workspace_focus_default_window (MetaWorkspace *workspace,
-                                     MetaWindow *not_this_one)
+                                     MetaWindow    *not_this_one,
+                                     Time           timestamp)
 {
+  if (timestamp == CurrentTime)
+    {
+      meta_warning ("CurrentTime used to choose focus window; "
+                    "focus window may not be correct.\n");
+    }
+
+
   if (meta_prefs_get_focus_mode () == META_FOCUS_MODE_CLICK)
-    meta_workspace_focus_mru_window (workspace, not_this_one);
+    meta_workspace_focus_mru_window (workspace, not_this_one, timestamp);
   else
     {
       MetaWindow * window;
@@ -801,22 +811,42 @@ meta_workspace_focus_default_window (MetaWorkspace *workspace,
           window->type != META_WINDOW_DOCK &&
           window->type != META_WINDOW_DESKTOP)
         {
-          meta_topic (META_DEBUG_FOCUS,
-                      "Focusing mouse window %s\n", window->desc);
+          if (timestamp == CurrentTime)
+            {
 
-          meta_window_focus (window, meta_display_get_current_time (workspace->screen->display));
+              /* We would like for this to never happen.  However, if
+               * it does happen then we kludge since using CurrentTime
+               * can mean ugly race conditions--and we can avoid these
+               * by allowing EnterNotify events (which come with
+               * timestamps) to handle focus.
+               */
+
+              meta_topic (META_DEBUG_FOCUS,
+                          "Not focusing mouse window %s because EnterNotify events should handle that\n", window->desc);
+            }
+          else
+            {
+              meta_topic (META_DEBUG_FOCUS,
+                          "Focusing mouse window %s\n", window->desc);
+              meta_window_focus (window, timestamp);
+            }
+
+          if (workspace->screen->display->autoraise_window != window &&
+              meta_prefs_get_auto_raise ()) 
+            {
+              meta_display_queue_autoraise_callback (workspace->screen->display,
+                                                     window);
+            }
         }
       else if (meta_prefs_get_focus_mode () == META_FOCUS_MODE_SLOPPY)
-        meta_workspace_focus_mru_window (workspace, not_this_one);
+        meta_workspace_focus_mru_window (workspace, not_this_one, timestamp);
       else if (meta_prefs_get_focus_mode () == META_FOCUS_MODE_MOUSE)
         {
           meta_topic (META_DEBUG_FOCUS,
                       "Setting focus to no_focus_window, since no valid "
                       "window to focus found.\n");
-          XSetInputFocus (workspace->screen->display->xdisplay,
-                          workspace->screen->display->no_focus_window,
-                          RevertToPointerRoot,
-                          meta_display_get_current_time (workspace->screen->display));
+          meta_display_focus_the_no_focus_window (workspace->screen->display,
+                                                  timestamp);
         }
     }
 }
@@ -824,7 +854,8 @@ meta_workspace_focus_default_window (MetaWorkspace *workspace,
 /* Focus MRU window (or top window if failed) on active workspace */
 void
 meta_workspace_focus_mru_window (MetaWorkspace *workspace,
-                                 MetaWindow *not_this_one)
+                                 MetaWindow    *not_this_one,
+                                 Time           timestamp)
 {
   MetaWindow *window = NULL;
   GList *tmp;
@@ -840,6 +871,7 @@ meta_workspace_focus_mru_window (MetaWorkspace *workspace,
       MetaWindow* tmp_window;
       tmp_window = ((MetaWindow*) tmp->data);
       if (tmp_window != not_this_one           &&
+          !tmp_window->minimized               &&
           tmp_window->type != META_WINDOW_DOCK &&
           tmp_window->type != META_WINDOW_DESKTOP)
         {
@@ -855,8 +887,7 @@ meta_workspace_focus_mru_window (MetaWorkspace *workspace,
       meta_topic (META_DEBUG_FOCUS,
                   "Focusing workspace MRU window %s\n", window->desc);
       
-      meta_window_focus (window,
-                         meta_display_get_current_time (workspace->screen->display));
+      meta_window_focus (window, timestamp);
 
       /* Also raise the window if in click-to-focus */
       if (meta_prefs_get_focus_mode () == META_FOCUS_MODE_CLICK)
@@ -864,41 +895,8 @@ meta_workspace_focus_mru_window (MetaWorkspace *workspace,
     }
   else
     {
-      meta_topic (META_DEBUG_FOCUS, "No MRU window to focus found\n");
-      meta_workspace_focus_top_window (workspace, not_this_one);
-    }
-}
-
-/* Focus top window on workspace */
-void
-meta_workspace_focus_top_window (MetaWorkspace *workspace,
-                                 MetaWindow    *not_this_one)
-{
-  MetaWindow *window;
-
-  if (not_this_one)
-    meta_topic (META_DEBUG_FOCUS,
-                "Focusing top window excluding %s\n", not_this_one->desc);
-  
-  window = meta_stack_get_default_focus_window (workspace->screen->stack,
-                                                workspace,
-                                                not_this_one);
-
-  /* FIXME I'm a loser on the CurrentTime front */
-  if (window)
-    {
-      meta_topic (META_DEBUG_FOCUS,
-                  "Focusing top window %s\n", window->desc);
-
-      meta_window_focus (window, 
-	                 meta_display_get_current_time (workspace->screen->display));
-
-      /* Also raise the window if in click-to-focus */
-      if (meta_prefs_get_focus_mode () == META_FOCUS_MODE_CLICK)
-        meta_window_raise (window);
-    }
-  else
-    {
-      meta_topic (META_DEBUG_FOCUS, "No top window to focus found\n");
+      meta_topic (META_DEBUG_FOCUS, "No MRU window to focus found; focusing no_focus_window.\n");
+      meta_display_focus_the_no_focus_window (workspace->screen->display,
+                                              timestamp);
     }
 }
