@@ -22,6 +22,7 @@
 #ifndef META_XPROPS_H
 #define META_XPROPS_H
 
+#include <config.h>
 #include "display.h"
 #include <X11/Xutil.h>
 
@@ -133,6 +134,7 @@ typedef enum
   META_PROP_VALUE_INVALID,
   META_PROP_VALUE_UTF8,
   META_PROP_VALUE_STRING,
+  META_PROP_VALUE_STRING_AS_UTF8,
   META_PROP_VALUE_MOTIF_HINTS,
   META_PROP_VALUE_CARDINAL,
   META_PROP_VALUE_WINDOW,
@@ -142,7 +144,8 @@ typedef enum
   META_PROP_VALUE_TEXT_PROPERTY, /* comes back as UTF-8 string */
   META_PROP_VALUE_WM_HINTS,
   META_PROP_VALUE_CLASS_HINT,
-  META_PROP_VALUE_SIZE_HINTS
+  META_PROP_VALUE_SIZE_HINTS,
+  META_PROP_VALUE_SYNC_COUNTER /* comes back as CARDINAL */
 } MetaPropValueType;
 
 /* used to request/return/store property values */
@@ -160,6 +163,9 @@ typedef struct
     gulong cardinal;
     XWMHints *wm_hints;
     XClassHint class_hint;
+#ifdef HAVE_XSYNC
+    XSyncCounter xcounter;
+#endif
     
     struct
     {
